@@ -18,7 +18,7 @@ from cola import qtutils
 from cola.compat import set
 from cola.dag import gravatar
 from cola.dag.model import RepoReader
-from cola.qtutils import tr
+from cola.i18n import N_
 from cola.widgets import completion
 from cola.widgets import defs
 from cola.widgets.createbranch import create_new_branch
@@ -92,6 +92,7 @@ class DiffInfoTask(QtCore.QRunnable):
 
 
 class DiffWidget(QtGui.QWidget):
+
     def __init__(self, notifier, parent):
         QtGui.QWidget.__init__(self, parent)
 
@@ -161,7 +162,7 @@ class DiffWidget(QtGui.QWidget):
             pass
 
     def set_diff_sha1(self, sha1):
-        self.diff.setText(self.tr('+++ Loading...'))
+        self.diff.setText('+++ ' + N_('Loading...'))
         task = DiffInfoTask(sha1, self.reflector)
         self.tasks.add(task)
         QtCore.QThreadPool.globalInstance().start(task)
@@ -198,6 +199,7 @@ class DiffWidget(QtGui.QWidget):
 
 class ViewerMixin(object):
     """Implementations must provide selected_items()"""
+
     def __init__(self):
         self.selected = None
         self.clicked = None
@@ -266,31 +268,31 @@ class ViewerMixin(object):
     def context_menu_actions(self):
         return {
         'diff_this_selected':
-            qtutils.add_action(self, 'Diff this -> selected',
+            qtutils.add_action(self, N_('Diff this -> selected'),
                                self.diff_this_selected),
         'diff_selected_this':
-            qtutils.add_action(self, 'Diff selected -> this',
+            qtutils.add_action(self, N_('Diff selected -> this'),
                                self.diff_selected_this),
         'create_branch':
-            qtutils.add_action(self, 'Create Branch',
+            qtutils.add_action(self, N_('Create Branch'),
                                self.create_branch),
         'create_patch':
-            qtutils.add_action(self, 'Create Patch',
+            qtutils.add_action(self, N_('Create Patch'),
                                self.create_patch),
         'create_tag':
-            qtutils.add_action(self, 'Create Tag',
+            qtutils.add_action(self, N_('Create Tag'),
                                self.create_tag),
         'create_tarball':
-            qtutils.add_action(self, 'Save As Tarball/Zip...',
+            qtutils.add_action(self, N_('Save As Tarball/Zip...'),
                                self.create_tarball),
         'cherry_pick':
-            qtutils.add_action(self, 'Cherry Pick',
+            qtutils.add_action(self, N_('Cherry Pick'),
                                self.cherry_pick),
         'save_blob':
-            qtutils.add_action(self, 'Grab File...',
+            qtutils.add_action(self, N_('Grab File...'),
                                self.save_blob_dialog),
         'copy':
-            qtutils.add_action(self, 'Copy SHA-1',
+            qtutils.add_action(self, N_('Copy SHA-1'),
                                self.copy_to_clipboard,
                                QtGui.QKeySequence.Copy),
         }
@@ -345,6 +347,7 @@ class ViewerMixin(object):
 
 
 class CommitTreeWidgetItem(QtGui.QTreeWidgetItem):
+
     def __init__(self, commit, parent=None):
         QtGui.QListWidgetItem.__init__(self, parent)
         self.commit = commit
@@ -353,7 +356,8 @@ class CommitTreeWidgetItem(QtGui.QTreeWidgetItem):
         self.setText(2, commit.authdate)
 
 
-class CommitTreeWidget(QtGui.QTreeWidget, ViewerMixin):
+class CommitTreeWidget(ViewerMixin, QtGui.QTreeWidget):
+
     def __init__(self, notifier, parent):
         QtGui.QTreeWidget.__init__(self, parent)
         ViewerMixin.__init__(self)
@@ -363,17 +367,17 @@ class CommitTreeWidget(QtGui.QTreeWidget, ViewerMixin):
         self.setAllColumnsShowFocus(True)
         self.setAlternatingRowColors(True)
         self.setRootIsDecorated(False)
-        self.setHeaderLabels([tr('Summary'), tr('Author'), tr('Age')])
+        self.setHeaderLabels([N_('Summary'), N_('Author'), N_('Age')])
 
         self.sha1map = {}
         self.notifier = notifier
         self.selecting = False
         self.commits = []
 
-        self.action_up = qtutils.add_action(self, 'Go Up', self.go_up,
+        self.action_up = qtutils.add_action(self, N_('Go Up'), self.go_up,
                                             Qt.Key_K)
 
-        self.action_down = qtutils.add_action(self, 'Go Down', self.go_down,
+        self.action_down = qtutils.add_action(self, N_('Go Down'), self.go_down,
                                               Qt.Key_J)
 
         notifier.add_observer(COMMITS_SELECTED, self.commits_selected)
@@ -480,7 +484,7 @@ class DAGView(Widget):
         Widget.__init__(self, parent)
 
         self.setAttribute(Qt.WA_MacMetalStyle)
-        self.setMinimumSize(1, 1)
+        self.setMinimumSize(420, 420)
 
         # change when widgets are added/removed
         self.widget_version = 1
@@ -494,7 +498,7 @@ class DAGView(Widget):
         self.old_ref = None
         self.thread = ReaderThread(dag, self)
 
-        self.revtext = completion.GitLogLineEdit(parent=self)
+        self.revtext = completion.GitLogLineEdit()
 
         self.maxresults = QtGui.QSpinBox()
         self.maxresults.setMinimum(1)
@@ -503,7 +507,7 @@ class DAGView(Widget):
         self.maxresults.setSuffix('')
 
         self.displaybutton = QtGui.QPushButton()
-        self.displaybutton.setText('Display')
+        self.displaybutton.setText(N_('Display'))
 
         self.zoom_in = QtGui.QPushButton()
         self.zoom_in.setIcon(qtutils.theme_icon('zoom-in.png'))
@@ -591,7 +595,7 @@ class DAGView(Widget):
         self.connect(self.maxresults, SIGNAL('editingFinished()'),
                      self.display)
 
-        self.connect(self.revtext, SIGNAL('ref_changed'),
+        self.connect(self.revtext, SIGNAL('changed()'),
                      self.display)
 
         self.connect(self.revtext, SIGNAL('textChanged(QString)'),
@@ -661,7 +665,7 @@ class DAGView(Widget):
         self.old_ref = new_ref
         self.old_count = new_count
 
-        self.setEnabled(False)
+        self.displaybutton.setEnabled(False)
         self.thread.stop()
         self.clear()
         self.dag.set_ref(new_ref)
@@ -694,7 +698,7 @@ class DAGView(Widget):
         self.treewidget.add_commits(commits)
 
     def thread_done(self):
-        self.setEnabled(True)
+        self.displaybutton.setEnabled(True)
         self.graphview.setFocus()
         try:
             commit_obj = self.commit_list[-1]
@@ -916,6 +920,7 @@ class EdgeColor(object):
     def current(cls):
         return cls.colors[cls.current_color_index]
 
+
 class Commit(QtGui.QGraphicsItem):
     item_type = QtGui.QGraphicsItem.UserType + 2
     commit_radius = 12.0
@@ -1119,8 +1124,7 @@ class Label(QtGui.QGraphicsItem):
             current_width += text_rect.width() + 5
 
 
-
-class GraphView(QtGui.QGraphicsView, ViewerMixin):
+class GraphView(ViewerMixin, QtGui.QGraphicsView):
 
     x_max = 0
     y_min = 0
@@ -1165,25 +1169,25 @@ class GraphView(QtGui.QGraphicsView, ViewerMixin):
         self.setResizeAnchor(QtGui.QGraphicsView.NoAnchor)
         self.setBackgroundBrush(QtGui.QColor(Qt.white))
 
-        qtutils.add_action(self, 'Zoom In',
+        qtutils.add_action(self, N_('Zoom In'),
                            self.zoom_in, Qt.Key_Plus, Qt.Key_Equal)
 
-        qtutils.add_action(self, 'Zoom Out',
+        qtutils.add_action(self, N_('Zoom Out'),
                            self.zoom_out, Qt.Key_Minus)
 
-        qtutils.add_action(self, 'Zoom to Fit',
+        qtutils.add_action(self, N_('Zoom to Fit'),
                            self.fit_view_to_selection, Qt.Key_F)
 
-        qtutils.add_action(self, 'Select Parent',
+        qtutils.add_action(self, N_('Select Parent'),
                            self.select_parent, 'Shift+J')
 
-        qtutils.add_action(self, 'Select Oldest Parent',
+        qtutils.add_action(self, N_('Select Oldest Parent'),
                            self.select_oldest_parent, Qt.Key_J)
 
-        qtutils.add_action(self, 'Select Child',
+        qtutils.add_action(self, N_('Select Child'),
                            self.select_child, 'Shift+K')
 
-        qtutils.add_action(self, 'Select Newest Child',
+        qtutils.add_action(self, N_('Select Newest Child'),
                            self.select_newest_child, Qt.Key_K)
 
         notifier.add_observer(COMMITS_SELECTED, self.commits_selected)
@@ -1197,7 +1201,7 @@ class GraphView(QtGui.QGraphicsView, ViewerMixin):
         self.y_min = 0
         self.commits = []
 
-    # ViewerMixin
+    # ViewerMixin interface
     def selected_items(self):
         """Return the currently selected items"""
         return self.scene().selectedItems()
