@@ -1,4 +1,5 @@
 import os
+import re
 
 import cola
 from cola import cmds
@@ -8,34 +9,33 @@ from cola import gitcmds
 from cola import qt
 from cola import qtutils
 from cola.git import git
+from cola.i18n import N_
 from cola.interaction import Interaction
 from cola.widgets.browse import BrowseDialog
-from cola.widgets.combodlg import ComboDialog
 from cola.widgets.grep import run_grep
 from cola.widgets.selectcommits import select_commits
 
 
-def choose_from_combo(title, items):
-    """Quickly choose an item from a list using a combo box"""
-    return ComboDialog(qtutils.active_window(), title=title, items=items).selected()
 
-
-def branch_delete():
+def delete_branch():
     """Launch the 'Delete Branch' dialog."""
-    branch = choose_from_combo('Delete Branch',
-                               cola.model().local_branches)
+    branch = choose_branch(N_('Delete Branch'), N_('Delete'))
     if not branch:
         return
     cmds.do(cmds.DeleteBranch, branch)
 
 
-def diff_revision():
-    """Diff an arbitrary revision against the worktree"""
-    ref = choose_ref('Select Revision to Diff', 'Diff',
-                     default='HEAD^')
-    if not ref:
+def delete_remote_branch():
+    """Launch the 'Delete Remote Branch' dialog."""
+    branch = choose_remote_branch(N_('Delete Remote Branch'), N_('Delete'))
+    if not branch:
         return
-    difftool.diff_commits(qtutils.active_window(), ref, None)
+    rgx = re.compile(r'^(?P<remote>[^/]+)/(?P<branch>.+)$')
+    match = rgx.match(branch)
+    if match:
+        remote = match.group('remote')
+        branch = match.group('branch')
+        cmds.do(cmds.DeleteRemoteBranch, remote, branch)
 
 
 def browse_current():
@@ -47,7 +47,7 @@ def browse_current():
 def browse_other():
     """Prompt for a branch and inspect content at that point in time."""
     # Prompt for a branch to browse
-    branch = choose_from_combo('Browse Revision...', gitcmds.all_refs())
+    branch = choose_ref(N_('Browse Commits...'), N_('Browse'))
     if not branch:
         return
     BrowseDialog.browse(branch)
@@ -55,8 +55,7 @@ def browse_other():
 
 def checkout_branch():
     """Launch the 'Checkout Branch' dialog."""
-    branch = choose_from_combo('Checkout Branch',
-                               cola.model().local_branches)
+    branch = choose_branch(N_('Checkout Branch'), N_('Checkout'))
     if not branch:
         return
     cmds.do(cmds.CheckoutBranch, branch)
@@ -65,7 +64,7 @@ def checkout_branch():
 def cherry_pick():
     """Launch the 'Cherry-Pick' dialog."""
     revs, summaries = gitcmds.log_helper(all=True)
-    commits = select_commits('Cherry-Pick Commit',
+    commits = select_commits(N_('Cherry-Pick Commit'),
                              revs, summaries, multiselect=False)
     if not commits:
         return
@@ -79,13 +78,13 @@ def clone_repo(spawn=True):
     A new cola session is invoked when 'spawn' is True.
 
     """
-    url, ok = qtutils.prompt('Path or URL to clone (Env. $VARS okay)')
+    url, ok = qtutils.prompt(N_('Path or URL to clone (Env. $VARS okay)'))
     url = os.path.expandvars(core.encode(url))
     if not ok or not url:
         return None
     try:
         # Pick a suitable basename by parsing the URL
-        newurl = url.replace('\\', '/')
+        newurl = url.replace('\\', '/').rstrip('/')
         default = newurl.rsplit('/', 1)[-1]
         if default == '.git':
             # The end of the URL is /.git, so assume it's a file path
@@ -100,13 +99,13 @@ def clone_repo(spawn=True):
             raise
     except:
         Interaction.information(
-                'Error Cloning',
-                'Could not parse: "%s"' % url)
-        Interaction.log('Oops, could not parse git url: "%s"' % url)
+                N_('Error Cloning'),
+                N_('Could not parse Git URL: "%s"') % url)
+        Interaction.log(N_('Could not parse Git URL: "%s"') % url)
         return None
 
     # Prompt the user for a directory to use as the parent directory
-    msg = 'Select a parent directory for the new clone'
+    msg = N_('Select a parent directory for the new clone')
     dirname = qtutils.opendir_dialog(msg, cola.model().getcwd())
     if not dirname:
         return None
@@ -116,7 +115,7 @@ def clone_repo(spawn=True):
     olddestdir = destdir
     if os.path.exists(destdir):
         # An existing path can be specified
-        msg = ('"%s" already exists, cola will create a new directory' %
+        msg = (N_('"%s" already exists, cola will create a new directory') %
                destdir)
         Interaction.information('Directory Exists', msg)
 
@@ -124,14 +123,15 @@ def clone_repo(spawn=True):
     while os.path.exists(destdir):
         destdir = olddestdir + str(count)
         count += 1
-    cmds.do(cmds.Clone, core.decode(url), destdir, spawn=spawn)
-    return destdir
+    if cmds.do(cmds.Clone, core.decode(url), destdir, spawn=spawn):
+        return destdir
+    return None
 
 
 def export_patches():
     """Run 'git format-patch' on a list of commits."""
     revs, summaries = gitcmds.log_helper()
-    to_export = select_commits('Export Patches', revs, summaries)
+    to_export = select_commits(N_('Export Patches'), revs, summaries)
     if not to_export:
         return
     to_export.reverse()
@@ -144,13 +144,9 @@ def diff_expression():
     tracked = gitcmds.tracked_branch()
     current = gitcmds.current_branch()
     if tracked and current:
-        default = tracked + '..' + current
+        ref = tracked + '..' + current
     else:
-        default = 'origin/master..'
-    ref = choose_ref('Enter Diff Expression', 'Diff',
-                     default=default)
-    if not ref:
-        return
+        ref = 'origin/master..'
     difftool.diff_expression(qtutils.active_window(), ref)
 
 
@@ -164,7 +160,7 @@ def grep():
 
 def open_repo():
     """Spawn a new cola session."""
-    dirname = qtutils.opendir_dialog('Open Git Repository...',
+    dirname = qtutils.opendir_dialog(N_('Open Git Repository...'),
                                      cola.model().getcwd())
     if not dirname:
         return
@@ -173,16 +169,35 @@ def open_repo():
 
 def load_commitmsg():
     """Load a commit message from a file."""
-    filename = qtutils.open_dialog('Load Commit Message...',
+    filename = qtutils.open_dialog(N_('Load Commit Message...'),
                                    cola.model().getcwd())
     if filename:
         cmds.do(cmds.LoadCommitMessage, filename)
 
 
+def choose_from_dialog(get, title, button_text, default):
+    parent = qtutils.active_window()
+    return get(title, button_text, parent, default=default)
+
+
+def choose_ref(title, button_text, default=None):
+    return choose_from_dialog(qt.GitRefDialog.get,
+                              title, button_text, default)
+
+
+def choose_branch(title, button_text, default=None):
+    return choose_from_dialog(qt.GitBranchDialog.get,
+                              title, button_text, default)
+
+
+def choose_remote_branch(title, button_text, default=None):
+    return choose_from_dialog(qt.GitRemoteBranchDialog.get,
+                              title, button_text, default)
+
+
 def rebase():
     """Rebase onto a branch."""
-    branch = choose_from_combo('Rebase Branch',
-                               cola.model().all_branches())
+    branch = choose_ref(N_('Select New Base'), N_('Rebase'))
     if not branch:
         return
     #TODO cmd
@@ -190,14 +205,9 @@ def rebase():
     Interaction.log_status(status, output, '')
 
 
-def choose_ref(title, button_text, default=None):
-    parent = qtutils.active_window()
-    return qt.GitRefDialog.ref(title, button_text, parent, default=default)
-
-
 def review_branch():
     """Diff against an arbitrary revision, branch, tag, etc."""
-    branch = choose_ref('Select Branch to Review', 'Review')
+    branch = choose_ref(N_('Select Branch to Review'), N_('Review'))
     if not branch:
         return
     merge_base = gitcmds.merge_base_parent(branch)
