@@ -1,6 +1,7 @@
 import cola
 from cola import gitcmds
 from cola import qtutils
+from cola.i18n import N_
 from cola.merge.view import MergeView
 
 
@@ -16,12 +17,12 @@ def local_merge():
 def abort_merge():
     """Prompts before aborting a merge in progress
     """
-    title = qtutils.tr('Abort Merge...')
-    txt = ('Aborting the current merge will cause '
-           '*ALL* uncommitted changes to be lost.\n'
-           'Recovering uncommitted changes is not possible.')
-    info_txt = 'Aborting the current merge?'
-    ok_txt = 'Abort Merge'
+    title = N_('Abort Merge...')
+    txt = N_('Aborting the current merge will cause '
+             '*ALL* uncommitted changes to be lost.\n'
+             'Recovering uncommitted changes is not possible.')
+    info_txt = N_('Aborting the current merge?')
+    ok_txt = N_('Abort Merge')
     if qtutils.confirm(title, txt, info_txt, ok_txt,
                        default=False, icon=qtutils.icon('undo.svg')):
         gitcmds.abort_merge()
