@@ -6,8 +6,11 @@ from PyQt4.QtCore import SIGNAL
 import cola
 from cola import cmds
 from cola import gitcmds
+from cola import gitcfg
 from cola import utils
 from cola.cmds import Interaction
+from cola.gitcmds import commit_message_path
+from cola.i18n import N_
 from cola.qt import create_toolbutton
 from cola.qtutils import add_action
 from cola.qtutils import confirm
@@ -15,7 +18,6 @@ from cola.qtutils import connect_action_bool
 from cola.qtutils import connect_button
 from cola.qtutils import options_icon
 from cola.qtutils import save_icon
-from cola.qtutils import tr
 from cola.widgets import defs
 from cola.prefs import diff_font
 from cola.prefs import tabwidth
@@ -25,7 +27,7 @@ from cola.dag.model import DAG
 from cola.dag.model import RepoReader
 from cola.widgets.selectcommits import select_commits
 from cola.widgets.text import HintedLineEdit
-from cola.widgets.text import HintedTextEdit
+from cola.widgets.spellcheck import SpellCheckTextEdit
 
 
 class CommitMessageEditor(QtGui.QWidget):
@@ -34,21 +36,23 @@ class CommitMessageEditor(QtGui.QWidget):
 
         self.model = model
         self.notifying = False
+        self.spellcheck_initialized = False
 
         self._linebreak = None
         self._textwidth = None
         self._tabwidth = None
 
         # Actions
-        self.signoff_action = add_action(self, cmds.SignOff.NAME,
+        self.signoff_action = add_action(self, cmds.SignOff.name(),
                                          cmds.run(cmds.SignOff),
                                          cmds.SignOff.SHORTCUT)
-        self.signoff_action.setToolTip('Sign off on this commit')
+        self.signoff_action.setToolTip(N_('Sign off on this commit'))
 
-        self.commit_action = add_action(self, 'Commit@@verb',
+        self.commit_action = add_action(self,
+                                        N_('Commit@@verb'),
                                         self.commit,
-                                        'Ctrl+Return')
-        self.commit_action.setToolTip(tr('Commit staged changes'))
+                                        cmds.Commit.SHORTCUT)
+        self.commit_action.setToolTip(N_('Commit staged changes'))
 
         # Widgets
         self.summary = CommitSummaryLineEdit()
@@ -59,14 +63,15 @@ class CommitMessageEditor(QtGui.QWidget):
         self.description.extra_actions.append(self.signoff_action)
         self.description.extra_actions.append(self.commit_action)
 
-        commit_button_tooltip = 'Commit staged changes\nShortcut: Ctrl+Enter'
-        self.commit_button = create_toolbutton(text='Commit@@verb',
+        commit_button_tooltip = N_('Commit staged changes\n'
+                                   'Shortcut: Ctrl+Enter')
+        self.commit_button = create_toolbutton(text=N_('Commit@@verb'),
                                                tooltip=commit_button_tooltip,
                                                icon=save_icon())
 
         self.actions_menu = QtGui.QMenu()
         self.actions_button = create_toolbutton(icon=options_icon(),
-                                                tooltip='Actions...')
+                                                tooltip=N_('Actions...'))
         self.actions_button.setMenu(self.actions_menu)
         self.actions_button.setPopupMode(QtGui.QToolButton.InstantPopup)
 
@@ -75,18 +80,27 @@ class CommitMessageEditor(QtGui.QWidget):
         self.actions_menu.addSeparator()
 
         # Amend checkbox
-        self.amend_action = self.actions_menu.addAction(tr('Amend Last Commit'))
+        self.amend_action = self.actions_menu.addAction(
+                N_('Amend Last Commit'))
         self.amend_action.setCheckable(True)
+        self.amend_action.setShortcut(cmds.AmendMode.SHORTCUT)
+        self.amend_action.setShortcutContext(Qt.ApplicationShortcut)
+
+        # Spell checker
+        self.check_spelling_action = self.actions_menu.addAction(
+                N_('Check Spelling'))
+        self.check_spelling_action.setCheckable(True)
+        self.check_spelling_action.setChecked(False)
 
         # Line wrapping
         self.actions_menu.addSeparator()
         self.autowrap_action = self.actions_menu.addAction(
-                tr('Auto-Wrap Lines'))
+                N_('Auto-Wrap Lines'))
         self.autowrap_action.setCheckable(True)
         self.autowrap_action.setChecked(linebreak())
 
         self.prev_commits_menu = self.actions_menu.addMenu(
-                tr('Load Previous Commit Message'))
+                N_('Load Previous Commit Message'))
         self.connect(self.prev_commits_menu, SIGNAL('aboutToShow()'),
                      self.build_prev_commits_menu)
 
@@ -111,19 +125,18 @@ class CommitMessageEditor(QtGui.QWidget):
 
         # Broadcast the amend mode
         connect_action_bool(self.amend_action, cmds.run(cmds.AmendMode))
+        connect_action_bool(self.check_spelling_action,
+                            self.toggle_check_spelling)
 
         # Handle the one-off autowrapping
         connect_action_bool(self.autowrap_action, self.set_linebreak)
 
+        add_action(self.summary, N_('Move Down'), self.focus_description,
+                Qt.Key_Down, Qt.Key_Return, Qt.Key_Enter)
+
         self.model.add_observer(self.model.message_commit_message_changed,
                                 self.set_commit_message)
 
-        self.connect(self.summary, SIGNAL('returnPressed()'),
-                     self.focus_description)
-
-        self.connect(self.summary, SIGNAL('downPressed()'),
-                     self.focus_description)
-
         self.connect(self.summary, SIGNAL('cursorPosition(int,int)'),
                      self.emit_position)
 
@@ -138,7 +151,7 @@ class CommitMessageEditor(QtGui.QWidget):
         self.connect(self.description, SIGNAL('textChanged()'),
                      self.commit_message_changed)
 
-        self.connect(self.description, SIGNAL('shiftTab()'),
+        self.connect(self.description, SIGNAL('leave()'),
                      self.focus_summary)
 
         self.setFont(diff_font())
@@ -155,6 +168,13 @@ class CommitMessageEditor(QtGui.QWidget):
         self.set_textwidth(textwidth())
         self.set_linebreak(linebreak())
 
+        # Loading message
+        commit_msg = ""
+        commit_msg_path = commit_message_path()
+        if commit_msg_path:
+            commit_msg = utils.slurp(commit_msg_path)
+        self.set_commit_message(commit_msg)
+
         # Allow tab to jump from the summary to the description
         self.setTabOrder(self.summary, self.description)
 
@@ -179,13 +199,13 @@ class CommitMessageEditor(QtGui.QWidget):
         else:
             description = self.formatted_description()
         if summary and description:
-            return summary + u'\n\n' + description
+            return summary + '\n\n' + description
         elif summary:
             return summary
         elif description:
-            return u'\n\n' + description
+            return '\n\n' + description
         else:
-            return u''
+            return ''
 
     def formatted_description(self):
         text = self.description.value()
@@ -227,13 +247,13 @@ class CommitMessageEditor(QtGui.QWidget):
 
         if num_lines == 0:
             # Message is empty
-            summary = u''
-            description = u''
+            summary = ''
+            description = ''
 
         elif num_lines == 1:
             # Message has a summary only
             summary = lines[0]
-            description = u''
+            description = ''
 
         elif num_lines == 2:
             # Message has two lines; this is not a common case
@@ -248,7 +268,7 @@ class CommitMessageEditor(QtGui.QWidget):
                 description_lines = lines[1:]
             else:
                 description_lines = lines[2:]
-            description = u'\n'.join(description_lines)
+            description = '\n'.join(description_lines)
 
         focus_summary = not summary
         focus_description = not description
@@ -318,49 +338,54 @@ class CommitMessageEditor(QtGui.QWidget):
         """Attempt to create a commit from the index and commit message."""
         if not bool(self.summary.value()):
             # Describe a good commit message
-            error_msg = tr(''
+            error_msg = N_(''
                 'Please supply a commit message.\n\n'
                 'A good commit message has the following format:\n\n'
                 '- First line: Describe in one sentence what you did.\n'
                 '- Second line: Blank\n'
                 '- Remaining lines: Describe why this change is good.\n')
             Interaction.log(error_msg)
-            Interaction.information('Missing Commit Message', error_msg)
+            Interaction.information(N_('Missing Commit Message'), error_msg)
             return
 
         msg = self.commit_message(raw=False)
 
         if not self.model.staged:
-            error_msg = tr(''
+            error_msg = N_(''
                 'No changes to commit.\n\n'
                 'You must stage at least 1 file before you can commit.')
             if self.model.modified:
-                informative_text = tr('Would you like to stage and '
+                informative_text = N_('Would you like to stage and '
                                       'commit all modified files?')
-                if not confirm('Stage and commit?',
+                if not confirm(N_('Stage and commit?'),
                                error_msg,
                                informative_text,
-                               'Stage and Commit',
+                               N_('Stage and Commit'),
                                default=False,
                                icon=save_icon()):
                     return
             else:
-                Interaction.information('Nothing to commit', error_msg)
+                Interaction.information(N_('Nothing to commit'), error_msg)
                 return
             cmds.do(cmds.StageModified)
 
         # Warn that amending published commits is generally bad
         amend = self.amend_action.isChecked()
         if (amend and self.model.is_commit_published() and
-            not confirm('Rewrite Published Commit?',
-                        'This commit has already been published.\n'
-                        'This operation will rewrite published history.\n'
-                        'You probably don\'t want to do this.',
-                        'Amend the published commit?',
-                        'Amend Commit',
+            not confirm(N_('Rewrite Published Commit?'),
+                        N_('This commit has already been published.\n'
+                           'This operation will rewrite published history.\n'
+                           'You probably don\'t want to do this.'),
+                        N_('Amend the published commit?'),
+                        N_('Amend Commit'),
                         default=False, icon=save_icon())):
             return
-        cmds.do(cmds.Commit, amend, msg)
+        status, output = cmds.do(cmds.Commit, amend, msg)
+        if status != 0:
+            Interaction.critical(N_('Commit failed'),
+                                 N_('"git commit" returned exit code %s') %
+                                    (status,),
+                                 output)
 
     def build_prev_commits_menu(self):
         dag = DAG('HEAD', 6)
@@ -380,38 +405,51 @@ class CommitMessageEditor(QtGui.QWidget):
 
         if len(commits) == 6:
             menu.addSeparator()
-            menu.addAction('More...', self.choose_commit)
+            menu.addAction(N_('More...'), self.choose_commit)
 
     def choose_commit(self):
         revs, summaries = gitcmds.log_helper()
-        sha1s = select_commits('Select Commit Message', revs, summaries,
+        sha1s = select_commits(N_('Select Commit Message'), revs, summaries,
                                multiselect=False)
         if not sha1s:
             return
         sha1 = sha1s[0]
         cmds.do(cmds.LoadPreviousMessage, sha1)
 
+    def toggle_check_spelling(self, enabled):
+        spellcheck = self.description.spellcheck
+
+        if enabled and not self.spellcheck_initialized:
+            # Add our name to the dictionary
+            self.spellcheck_initialized = True
+            cfg = gitcfg.instance()
+            user_name = cfg.get('user.name')
+            if user_name:
+                for part in user_name.split():
+                    spellcheck.add_word(part)
+
+            # Add our email address to the dictionary
+            user_email = cfg.get('user.email')
+            if user_email:
+                for part in user_email.split('@'):
+                    for elt in part.split('.'):
+                        spellcheck.add_word(elt)
+
+            # git jargon
+            spellcheck.add_word('Acked')
+            spellcheck.add_word('Signed')
+            spellcheck.add_word('Closes')
+            spellcheck.add_word('Fixes')
+
+        self.description.highlighter.enable(enabled)
+
 
 class CommitSummaryLineEdit(HintedLineEdit):
     def __init__(self, parent=None):
-        hint = u'Commit summary'
+        hint = N_('Commit summary')
         HintedLineEdit.__init__(self, hint, parent)
         self.extra_actions = []
 
-    def keyPressEvent(self, event):
-        if event.key() == Qt.Key_Down:
-            position = self.cursorPosition()
-            curtext = unicode(self.text())
-            if position == len(curtext):
-                self.emit(SIGNAL('downPressed()'))
-                event.accept()
-                return
-        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
-            self.emit(SIGNAL('returnPressed()'))
-            event.accept()
-            return
-        HintedLineEdit.keyPressEvent(self, event)
-
     def contextMenuEvent(self, event):
         menu = self.createStandardContextMenu()
         if self.extra_actions:
@@ -421,15 +459,15 @@ class CommitSummaryLineEdit(HintedLineEdit):
         menu.exec_(self.mapToGlobal(event.pos()))
 
 
-class CommitMessageTextEdit(HintedTextEdit):
+class CommitMessageTextEdit(SpellCheckTextEdit):
     def __init__(self, parent=None):
-        hint = u'Extended description...'
-        HintedTextEdit.__init__(self, hint, parent)
+        hint = N_('Extended description...')
+        SpellCheckTextEdit.__init__(self, hint, parent)
         self.extra_actions = []
         self.setMinimumSize(QtCore.QSize(1, 1))
 
-        self.action_emit_shift_tab = add_action(self,
-                'Shift Tab', self.emit_shift_tab, 'Shift+tab')
+        self.action_emit_leave = add_action(self,
+                'Shift Tab', self.emit_leave, 'Shift+tab')
 
         self.installEventFilter(self)
 
@@ -445,7 +483,7 @@ class CommitMessageTextEdit(HintedTextEdit):
         return False
 
     def contextMenuEvent(self, event):
-        menu = self.createStandardContextMenu()
+        menu, spell_menu = self.context_menu()
         if self.extra_actions:
             menu.addSeparator()
         for action in self.extra_actions:
@@ -457,16 +495,23 @@ class CommitMessageTextEdit(HintedTextEdit):
             cursor = self.textCursor()
             position = cursor.position()
             if position == 0:
+                # The cursor is at the beginning of the line.
+                # If we have selection then simply reset the cursor.
+                # Otherwise, emit a signal so that the parent can
+                # change focus.
                 if cursor.hasSelection():
                     cursor.setPosition(0)
                     self.setTextCursor(cursor)
                 else:
-                    self.emit_shift_tab()
+                    self.emit_leave()
                 event.accept()
                 return
             text_before = unicode(self.toPlainText())[:position]
             lines_before = text_before.count('\n')
             if lines_before == 0:
+                # If we're on the first line, but not at the
+                # beginning, then move the cursor to the beginning
+                # of the line.
                 if event.modifiers() & Qt.ShiftModifier:
                     mode = QtGui.QTextCursor.KeepAnchor
                 else:
@@ -490,7 +535,7 @@ class CommitMessageTextEdit(HintedTextEdit):
                 self.setTextCursor(cursor)
                 event.accept()
                 return
-        HintedTextEdit.keyPressEvent(self, event)
+        SpellCheckTextEdit.keyPressEvent(self, event)
 
-    def emit_shift_tab(self):
-        self.emit(SIGNAL('shiftTab()'))
+    def emit_leave(self):
+        self.emit(SIGNAL('leave()'))
