@@ -9,6 +9,7 @@ from cola import utils
 from cola import qtutils
 from cola.cmds import do
 from cola.git import git
+from cola.i18n import N_
 from cola.prefs import diff_font
 from cola.widgets import defs
 from cola.widgets.standard import Dialog
@@ -47,26 +48,30 @@ class Grep(Dialog):
         self.input_label = QtGui.QLabel('git grep')
         self.input_label.setFont(diff_font())
 
-        hint = 'command-line arguments'
-        self.input_txt = GrepLineEdit(hint, self)
+        hint = N_('command-line arguments')
+        self.input_txt = HintedLineEdit(hint, self)
         self.input_txt.enable_hint(True)
 
-        hint = 'grep result...'
+        hint = N_('grep result...')
         self.result_txt = GrepTextView(hint, self)
         self.result_txt.enable_hint(True)
 
-        self.edit_button = QtGui.QPushButton(self.tr('Edit'))
+        self.edit_button = QtGui.QPushButton(N_('Edit'))
         self.edit_button.setIcon(qtutils.open_file_icon())
         self.edit_button.setEnabled(False)
         self.edit_button.setShortcut(cmds.Edit.SHORTCUT)
 
-        self.shell_checkbox = QtGui.QCheckBox(self.tr('Shell arguments'))
+        self.refresh_button = QtGui.QPushButton(N_('Refresh'))
+        self.refresh_button.setIcon(qtutils.reload_icon())
+        self.refresh_button.setShortcut(QtGui.QKeySequence.Refresh)
+
+        self.shell_checkbox = QtGui.QCheckBox(N_('Shell arguments'))
         self.shell_checkbox.setToolTip(
-                'Parse arguments using a shell.\n'
-                'Queries with spaces will require "double quotes".')
+                N_('Parse arguments using a shell.\n'
+                   'Queries with spaces will require "double quotes".'))
         self.shell_checkbox.setChecked(False)
 
-        self.close_button = QtGui.QPushButton(self.tr('Close'))
+        self.close_button = QtGui.QPushButton(N_('Close'))
 
         self.input_layout = QtGui.QHBoxLayout()
         self.input_layout.setMargin(0)
@@ -84,6 +89,7 @@ class Grep(Dialog):
         self.input_layout.addWidget(self.input_txt)
 
         self.bottom_layout.addWidget(self.edit_button)
+        self.bottom_layout.addWidget(self.refresh_button)
         self.bottom_layout.addWidget(self.shell_checkbox)
         self.bottom_layout.addStretch()
         self.bottom_layout.addWidget(self.close_button)
@@ -101,10 +107,14 @@ class Grep(Dialog):
         self.connect(self.input_txt, SIGNAL('textChanged(QString)'),
                      self.input_txt_changed)
 
-        self.connect(self.input_txt, SIGNAL('returnPressed()'),
-                     lambda: self.result_txt.setFocus())
+        self.connect(self.result_txt, SIGNAL('leave()'),
+                     lambda: self.input_txt.setFocus())
 
+        qtutils.add_action(self.input_txt, 'FocusResults',
+                           lambda: self.result_txt.setFocus(),
+                           Qt.Key_Down, Qt.Key_Enter, Qt.Key_Return)
         qtutils.connect_button(self.edit_button, self.edit)
+        qtutils.connect_button(self.refresh_button, self.search)
         qtutils.connect_button(self.close_button, self.close)
         qtutils.add_close_action(self)
 
@@ -122,6 +132,7 @@ class Grep(Dialog):
 
     def search(self):
         self.edit_button.setEnabled(False)
+        self.refresh_button.setEnabled(False)
 
         self.grep_thread.txt = self.input_txt.as_unicode()
         self.grep_thread.shell = self.shell_checkbox.isChecked()
@@ -129,27 +140,23 @@ class Grep(Dialog):
 
     def search_for(self, txt):
         self.input_txt.set_value(txt)
-        self.run()
+        self.search()
+
+    def process_result(self, status, output):
+        if status == 0:
+            self.result_txt.set_value(output)
+        elif output:
+            self.result_txt.set_value('git grep: ' + output)
+        else:
+            self.result_txt.set_value('')
 
-    def process_result(self, status, out):
-        self.result_txt.set_value(out)
         self.edit_button.setEnabled(status == 0)
+        self.refresh_button.setEnabled(status == 0)
 
     def edit(self):
         goto_grep(self.result_txt.selected_line()),
 
 
-class GrepLineEdit(HintedLineEdit):
-    def __init__(self, hint, parent):
-        HintedLineEdit.__init__(self, hint, parent)
-
-    def keyPressEvent(self, event):
-        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
-            self.emit(SIGNAL('returnPressed()'))
-        else:
-            HintedLineEdit.keyPressEvent(self, event)
-
-
 class GrepTextView(HintedTextView):
     def __init__(self, hint, parent):
         HintedTextView.__init__(self, hint, parent)
@@ -232,6 +239,18 @@ class GrepTextView(HintedTextView):
             painter = QtGui.QPainter(self.viewport())
             painter.fillRect(rect, Qt.SolidPattern)
 
+    def keyPressEvent(self, event):
+        if event.key() == Qt.Key_Up:
+            cursor = self.textCursor()
+            position = cursor.position()
+            if position == 0 and not cursor.hasSelection():
+                # The cursor is at the beginning of the line.
+                # If we have selection then simply reset the cursor.
+                # Otherwise, emit a signal so that the parent can
+                # change focus.
+                self.emit(SIGNAL('leave()'))
+        return HintedTextView.keyPressEvent(self, event)
+
 
 def goto_grep(line):
     """Called when Search -> Grep's right-click 'goto' action."""
