@@ -1,5 +1,4 @@
 import os
-import sys
 import subprocess
 import itertools
 
@@ -13,20 +12,11 @@ from cola import cmds
 from cola import qtutils
 from cola import utils
 from cola.compat import set
+from cola.i18n import N_
 from cola.interaction import Interaction
 from cola.models.selection import State
 
 
-def select_item(tree, item):
-    if not item:
-        return
-    tree.setItemSelected(item, True)
-    parent = item.parent()
-    if parent:
-        tree.scrollToItem(parent)
-    tree.scrollToItem(item)
-
-
 class StatusWidget(QtGui.QWidget):
     """
     Provides a git-status-like repository widget.
@@ -51,6 +41,9 @@ class StatusWidget(QtGui.QWidget):
     def restore_size(self):
         self.setMaximumWidth(2 ** 13)
 
+    def refresh(self):
+        self.tree.show_selection()
+
 
 class StatusTreeWidget(QtGui.QTreeWidget):
     # Item categories
@@ -61,8 +54,6 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     idx_untracked = 3
     idx_end = 4
 
-    txt_parent_dir = 'Open Parent Directory'
-
     # Read-only access to the mode state
     mode = property(lambda self: self.m.mode)
 
@@ -78,30 +69,31 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.setRootIsDecorated(False)
         self.setIndentation(0)
 
-        self.add_item('Staged', hide=True)
-        self.add_item('Unmerged', hide=True)
-        self.add_item('Modified', hide=True)
-        self.add_item('Untracked', hide=True)
+        self.add_item(N_('Staged'), hide=True)
+        self.add_item(N_('Unmerged'), hide=True)
+        self.add_item(N_('Modified'), hide=True)
+        self.add_item(N_('Untracked'), hide=True)
 
         # Used to restore the selection
         self.old_scroll = None
         self.old_selection = None
         self.old_contents = None
+        self.old_current_item = None
 
         self.expanded_items = set()
 
         self.process_selection = qtutils.add_action(self,
-                'Stage/Unstage', self._process_selection,
+                N_('Stage / Unstage'), self._process_selection,
                 cmds.Stage.SHORTCUT)
 
         self.launch_difftool = qtutils.add_action(self,
-                self.tr(cmds.LaunchDifftool.NAME),
+                cmds.LaunchDifftool.name(),
                 cmds.run(cmds.LaunchDifftool),
                 cmds.LaunchDifftool.SHORTCUT)
         self.launch_difftool.setIcon(qtutils.icon('git.svg'))
 
         self.launch_editor = qtutils.add_action(self,
-                self.tr(cmds.LaunchEditor.NAME),
+                cmds.LaunchEditor.name(),
                 cmds.run(cmds.LaunchEditor),
                 cmds.LaunchEditor.SHORTCUT,
                 'Return', 'Enter')
@@ -109,26 +101,26 @@ class StatusTreeWidget(QtGui.QTreeWidget):
 
         if not utils.is_win32():
             self.open_using_default_app = qtutils.add_action(self,
-                    self.tr(cmds.OpenDefaultApp.NAME),
+                    cmds.OpenDefaultApp.name(),
                     self._open_using_default_app,
                     cmds.OpenDefaultApp.SHORTCUT)
             self.open_using_default_app.setIcon(qtutils.file_icon())
 
             self.open_parent_dir = qtutils.add_action(self,
-                    self.tr(cmds.OpenParentDir.NAME),
+                    cmds.OpenParentDir.name(),
                     self._open_parent_dir,
                     cmds.OpenParentDir.SHORTCUT)
             self.open_parent_dir.setIcon(qtutils.open_file_icon())
 
-        self.up = qtutils.add_action(self,
-                'Move Up', self.move_up, Qt.Key_K)
-        self.down = qtutils.add_action(self,
-                'Move Down', self.move_down, Qt.Key_J)
+        self.up = qtutils.add_action(self, N_('Move Up'), self.move_up,
+                                     Qt.Key_K)
+
+        self.down = qtutils.add_action(self, N_('Move Down'), self.move_down,
+                                       Qt.Key_J)
 
         self.copy_path_action = qtutils.add_action(self,
-                                                   'Copy Path to Clipboard',
-                                                   self.copy_path,
-                                                   QtGui.QKeySequence.Copy)
+                N_('Copy Path to Clipboard'),
+                self.copy_path, QtGui.QKeySequence.Copy)
         self.copy_path_action.setIcon(qtutils.theme_icon('edit-copy.svg'))
 
         self.connect(self, SIGNAL('about_to_update'), self._about_to_update)
@@ -163,7 +155,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
 
         item = QtGui.QTreeWidgetItem(self)
         item.setFont(0, font)
-        item.setText(0, self.tr(txt))
+        item.setText(0, txt)
         if hide:
             self.setItemHidden(item, True)
 
@@ -175,50 +167,90 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         old_s = self.old_selection
         new_c = self.contents()
 
-        def select_modified(item):
-            idx = new_c.modified.index(item)
-            select_item(self, self.modified_item(idx))
+        def mkselect(lst, widget_getter):
+            def select(item, current=False):
+                idx = lst.index(item)
+                widget = widget_getter(idx)
+                if current:
+                    self.setCurrentItem(widget)
+                self.setItemSelected(widget, True)
+            return select
 
-        def select_unmerged(item):
-            idx = new_c.unmerged.index(item)
-            select_item(self, self.unmerged_item(idx))
+        select_staged = mkselect(new_c.staged, self.staged_item)
+        select_unmerged = mkselect(new_c.unmerged, self.unmerged_item)
+        select_modified = mkselect(new_c.modified, self.modified_item)
+        select_untracked = mkselect(new_c.untracked, self.untracked_item)
 
-        def select_untracked(item):
-            idx = new_c.untracked.index(item)
-            select_item(self, self.untracked_item(idx))
+        saved_selection = [
+        (set(new_c.staged), old_c.staged, set(old_s.staged),
+            select_staged),
 
-        def select_staged(item):
-            idx = new_c.staged.index(item)
-            select_item(self, self.staged_item(idx))
+        (set(new_c.unmerged), old_c.unmerged, set(old_s.unmerged),
+            select_unmerged),
 
-        restore_selection_actions = (
-            (new_c.modified, old_c.modified, old_s.modified, select_modified),
-            (new_c.unmerged, old_c.unmerged, old_s.unmerged, select_unmerged),
-            (new_c.untracked, old_c.untracked, old_s.untracked, select_untracked),
-            (new_c.staged, old_c.staged, old_s.staged, select_staged),
-        )
+        (set(new_c.modified), old_c.modified, set(old_s.modified),
+            select_modified),
 
-        for (new, old, selection, action) in restore_selection_actions:
+        (set(new_c.untracked), old_c.untracked, set(old_s.untracked),
+            select_untracked),
+        ]
+
+        # Restore the current item
+        if self.old_current_item:
+            category, idx = self.old_current_item
+            if category == self.idx_header:
+                item = self.invisibleRootItem().child(idx)
+                self.setCurrentItem(item)
+                self.setItemSelected(item, True)
+                return
+            # Reselect the current item
+            selection_info = saved_selection[category]
+            new = selection_info[0]
+            old = selection_info[1]
+            reselect = selection_info[3]
+            try:
+                item = old[idx]
+            except:
+                return
+            if item in new:
+                reselect(item, current=True)
+
+        # Restore selection
+        # When reselecting we only care that the items are selected;
+        # we do not need to rerun the callbacks which were triggered
+        # above.  Block signals to skip the callbacks.
+        self.blockSignals(True)
+        for (new, old, selection, reselect) in saved_selection:
+            for item in selection:
+                if item in new:
+                    reselect(item, current=False)
+        self.blockSignals(False)
+
+        for (new, old, selection, reselect) in saved_selection:
             # When modified is staged, select the next modified item
             # When unmerged is staged, select the next unmerged item
-            # When untracked is staged, select the next untracked item
-            # When something is unstaged we should select the next staged item
-            new_set = set(new)
-            if len(new) < len(old) and old:
-                for idx, i in enumerate(old):
-                    if i not in new_set:
-                        for j in itertools.chain(old[idx+1:],
-                                                 reversed(old[:idx])):
-                            if j in new_set:
-                                action(j)
-                                return
-
-        for (new, old, selection, action) in restore_selection_actions:
-            # Reselect items when doing partial-staging
-            new_set = set(new)
+            # When unstaging, select the next staged item
+            # When staging untracked files, select the next untracked item
+            if len(new) >= len(old):
+                # The list did not shrink so it is not one of these cases.
+                continue
             for item in selection:
-                if item in new_set:
-                    action(item)
+                # The item still exists so ignore it
+                if item in new or item not in old:
+                    continue
+                # The item no longer exists in this list so search for
+                # its nearest neighbors and select them instead.
+                idx = old.index(item)
+                for j in itertools.chain(old[idx+1:], reversed(old[:idx])):
+                    if j in new:
+                        reselect(j, current=True)
+                        return
+
+    def restore_scrollbar(self):
+        vscroll = self.verticalScrollBar()
+        if vscroll and self.old_scroll is not None:
+            vscroll.setValue(self.old_scroll)
+            self.old_scroll = None
 
     def staged_item(self, itemidx):
         return self._subtree_item(self.idx_staged, itemidx)
@@ -259,13 +291,32 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.emit(SIGNAL('about_to_update'))
 
     def _about_to_update(self):
-        self.old_selection = self.selection()
-        self.old_contents = self.contents()
+        self.save_selection()
+        self.save_scrollbar()
 
-        self.old_scroll = None
+    def save_scrollbar(self):
         vscroll = self.verticalScrollBar()
         if vscroll:
             self.old_scroll = vscroll.value()
+        else:
+            self.old_scroll = None
+
+    def current_item(self):
+        current = self.currentItem()
+        if not current:
+            return None
+        idx = self.indexFromItem(current, 0)
+        if idx.parent().isValid():
+            parent_idx = idx.parent()
+            entry = (parent_idx.row(), idx.row())
+        else:
+            entry = (self.idx_header, idx.row())
+        return entry
+
+    def save_selection(self):
+        self.old_contents = self.contents()
+        self.old_selection = self.selection()
+        self.old_current_item = self.current_item()
 
     def updated(self):
         """Update display from model data."""
@@ -277,12 +328,8 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         self.set_unmerged(self.m.unmerged)
         self.set_untracked(self.m.untracked)
 
-        vscroll = self.verticalScrollBar()
-        if vscroll and self.old_scroll is not None:
-            vscroll.setValue(self.old_scroll)
-            self.old_scroll = None
-
         self.restore_selection()
+        self.restore_scrollbar()
         self.update_column_widths()
 
     def set_staged(self, items):
@@ -368,25 +415,25 @@ class StatusTreeWidget(QtGui.QTreeWidget):
     def _create_header_context_menu(self, menu, idx):
         if idx == self.idx_staged:
             menu.addAction(qtutils.icon('remove.svg'),
-                           self.tr('Unstage All'),
+                           N_('Unstage All'),
                            cmds.run(cmds.UnstageAll))
             return menu
         elif idx == self.idx_unmerged:
             action = menu.addAction(qtutils.icon('add.svg'),
-                                    self.tr(cmds.StageUnmerged.NAME),
+                                    cmds.StageUnmerged.name(),
                                     cmds.run(cmds.StageUnmerged))
             action.setShortcut(cmds.StageUnmerged.SHORTCUT)
             return menu
         elif idx == self.idx_modified:
             action = menu.addAction(qtutils.icon('add.svg'),
-                                    self.tr(cmds.StageModified.NAME),
+                                    cmds.StageModified.name(),
                                     cmds.run(cmds.StageModified))
             action.setShortcut(cmds.StageModified.SHORTCUT)
             return menu
 
         elif idx == self.idx_untracked:
             action = menu.addAction(qtutils.icon('add.svg'),
-                                    self.tr(cmds.StageUntracked.NAME),
+                                    cmds.StageUntracked.name(),
                                     cmds.run(cmds.StageUntracked))
             action.setShortcut(cmds.StageUntracked.SHORTCUT)
             return menu
@@ -396,57 +443,61 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             return self._create_staged_submodule_context_menu(menu, s)
 
         action = menu.addAction(qtutils.options_icon(),
-                                self.tr(cmds.LaunchEditor.NAME),
+                                cmds.LaunchEditor.name(),
                                 cmds.run(cmds.LaunchEditor))
         action.setShortcut(cmds.LaunchEditor.SHORTCUT)
 
         action = menu.addAction(qtutils.git_icon(),
-                                self.tr(cmds.LaunchDifftool.NAME),
+                                cmds.LaunchDifftool.name(),
                                 cmds.run(cmds.LaunchDifftool))
         action.setShortcut(cmds.LaunchDifftool.SHORTCUT)
 
         if self.m.unstageable():
             menu.addSeparator()
             action = menu.addAction(qtutils.icon('remove.svg'),
-                                    self.tr('Unstage Selected'),
+                                    N_('Unstage Selected'),
                                     cmds.run(cmds.Unstage, self.staged()))
             action.setShortcut(cmds.Unstage.SHORTCUT)
 
         if not utils.is_win32():
             menu.addSeparator()
             action = menu.addAction(qtutils.file_icon(),
-                    self.tr(cmds.OpenDefaultApp.NAME),
+                    cmds.OpenDefaultApp.name(),
                     cmds.run(cmds.OpenDefaultApp, self.staged()))
             action.setShortcut(cmds.OpenDefaultApp.SHORTCUT)
 
             action = menu.addAction(qtutils.open_file_icon(),
-                    self.tr(cmds.OpenParentDir.NAME),
+                    cmds.OpenParentDir.name(),
                     self._open_parent_dir)
             action.setShortcut(cmds.OpenParentDir.SHORTCUT)
 
         if self.m.undoable():
             menu.addSeparator()
             menu.addAction(qtutils.icon('undo.svg'),
-                           self.tr('Revert Unstaged Edits...'),
+                           N_('Revert Unstaged Edits...'),
                            lambda: self._revert_unstaged_edits(staged=True))
+            menu.addAction(qtutils.icon('undo.svg'),
+                           N_('Revert Uncommited Edits...'),
+                           lambda: self._revert_uncommitted_edits(
+                                        self.staged()))
         menu.addSeparator()
         menu.addAction(self.copy_path_action)
         return menu
 
     def _create_staged_submodule_context_menu(self, menu, s):
         menu.addAction(qtutils.git_icon(),
-                       self.tr('Launch git-cola'),
+                       N_('Launch git-cola'),
                        cmds.run(cmds.OpenRepo,
                                 os.path.abspath(s.staged[0])))
 
         action = menu.addAction(qtutils.options_icon(),
-                                self.tr(cmds.LaunchEditor.NAME),
+                                cmds.LaunchEditor.name(),
                                 cmds.run(cmds.LaunchEditor))
         action.setShortcut(cmds.LaunchEditor.SHORTCUT)
 
         menu.addSeparator()
         action = menu.addAction(qtutils.icon('remove.svg'),
-                                self.tr('Unstage Selected'),
+                                N_('Unstage Selected'),
                                 cmds.run(cmds.Unstage, self.staged()))
         action.setShortcut(cmds.Unstage.SHORTCUT)
 
@@ -456,28 +507,28 @@ class StatusTreeWidget(QtGui.QTreeWidget):
 
     def _create_unmerged_context_menu(self, menu, s):
         menu.addAction(qtutils.git_icon(),
-                       self.tr('Launch Merge Tool'),
+                       N_('Launch Merge Tool'),
                        cmds.run(cmds.Mergetool, self.unmerged()))
 
         action = menu.addAction(qtutils.icon('add.svg'),
-                                self.tr('Stage Selected'),
+                                N_('Stage Selected'),
                                 cmds.run(cmds.Stage, self.unstaged()))
         action.setShortcut(cmds.Stage.SHORTCUT)
         menu.addSeparator()
         action = menu.addAction(qtutils.options_icon(),
-                                self.tr(cmds.LaunchEditor.NAME),
+                                cmds.LaunchEditor.name(),
                                 cmds.run(cmds.LaunchEditor))
         action.setShortcut(cmds.LaunchEditor.SHORTCUT)
 
         if not utils.is_win32():
             menu.addSeparator()
             action = menu.addAction(qtutils.file_icon(),
-                    self.tr(cmds.OpenDefaultApp.NAME),
+                    cmds.OpenDefaultApp.name(),
                     cmds.run(cmds.OpenDefaultApp, self.unmerged()))
             action.setShortcut(cmds.OpenDefaultApp.SHORTCUT)
 
             action = menu.addAction(qtutils.open_file_icon(),
-                    self.tr(cmds.OpenParentDir.NAME),
+                    cmds.OpenParentDir.name(),
                     self._open_parent_dir)
             action.setShortcut(cmds.OpenParentDir.SHORTCUT)
 
@@ -493,20 +544,20 @@ class StatusTreeWidget(QtGui.QTreeWidget):
 
         if self.unstaged():
             action = menu.addAction(qtutils.options_icon(),
-                                    self.tr(cmds.LaunchEditor.NAME),
+                                    cmds.LaunchEditor.name(),
                                     cmds.run(cmds.LaunchEditor))
             action.setShortcut(cmds.Edit.SHORTCUT)
 
         if s.modified and self.m.stageable():
             action = menu.addAction(qtutils.git_icon(),
-                                    self.tr(cmds.LaunchDifftool.NAME),
+                                    cmds.LaunchDifftool.name(),
                                     cmds.run(cmds.LaunchDifftool))
             action.setShortcut(cmds.LaunchDifftool.SHORTCUT)
 
         if self.m.stageable():
             menu.addSeparator()
             action = menu.addAction(qtutils.icon('add.svg'),
-                                    self.tr('Stage Selected'),
+                                    N_('Stage Selected'),
                                     cmds.run(cmds.Stage, self.unstaged()))
             action.setShortcut(cmds.Stage.SHORTCUT)
 
@@ -514,31 +565,32 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             if self.m.undoable():
                 menu.addSeparator()
                 menu.addAction(qtutils.icon('undo.svg'),
-                               self.tr('Revert Unstaged Edits...'),
+                               N_('Revert Unstaged Edits...'),
                                self._revert_unstaged_edits)
                 menu.addAction(qtutils.icon('undo.svg'),
-                               self.tr('Revert Uncommited Edits...'),
-                               self._revert_uncommitted_edits)
+                               N_('Revert Uncommited Edits...'),
+                               lambda: self._revert_uncommitted_edits(
+                                            self.modified()))
 
         if self.unstaged() and not utils.is_win32():
             menu.addSeparator()
             action = menu.addAction(qtutils.file_icon(),
-                    self.tr(cmds.OpenDefaultApp.NAME),
+                    cmds.OpenDefaultApp.name(),
                     cmds.run(cmds.OpenDefaultApp, self.unstaged()))
             action.setShortcut(cmds.OpenDefaultApp.SHORTCUT)
 
             action = menu.addAction(qtutils.open_file_icon(),
-                    self.tr(cmds.OpenParentDir.NAME),
+                    cmds.OpenParentDir.name(),
                     self._open_parent_dir)
             action.setShortcut(cmds.OpenParentDir.SHORTCUT)
 
         if s.untracked:
             menu.addSeparator()
             menu.addAction(qtutils.discard_icon(),
-                           self.tr('Delete File(s)...'), self._delete_files)
+                           N_('Delete File(s)...'), self._delete_files)
             menu.addSeparator()
             menu.addAction(qtutils.icon('edit-clear.svg'),
-                           self.tr('Add to .gitignore'),
+                           N_('Add to .gitignore'),
                            cmds.run(cmds.Ignore,
                                 map(lambda x: '/' + x, self.untracked())))
         menu.addSeparator()
@@ -547,19 +599,19 @@ class StatusTreeWidget(QtGui.QTreeWidget):
 
     def _create_modified_submodule_context_menu(self, menu, s):
         menu.addAction(qtutils.git_icon(),
-                       self.tr('Launch git-cola'),
+                       N_('Launch git-cola'),
                        cmds.run(cmds.OpenRepo,
                             os.path.abspath(s.modified[0])))
 
         action = menu.addAction(qtutils.options_icon(),
-                                self.tr(cmds.LaunchEditor.NAME),
+                                cmds.LaunchEditor.name(),
                                 cmds.run(cmds.LaunchEditor))
         action.setShortcut(cmds.Edit.SHORTCUT)
 
         if self.m.stageable():
             menu.addSeparator()
             action = menu.addAction(qtutils.icon('add.svg'),
-                                    self.tr('Stage Selected'),
+                                    N_('Stage Selected'),
                                     cmds.run(cmds.Stage, self.unstaged()))
             action.setShortcut(cmds.Stage.SHORTCUT)
 
@@ -574,16 +626,16 @@ class StatusTreeWidget(QtGui.QTreeWidget):
         if count == 0:
             return
 
-        title = 'Delete Files?'
-        msg = self.tr('The following files will be deleted:\n\n')
+        title = N_('Delete Files?')
+        msg = N_('The following files will be deleted:') + '\n\n'
 
         fileinfo = subprocess.list2cmdline(files)
         if len(fileinfo) > 2048:
             fileinfo = fileinfo[:2048].rstrip() + '...'
         msg += fileinfo
 
-        info_txt = unicode(self.tr('Delete %d file(s)?')) % count
-        ok_txt = 'Delete Files'
+        info_txt = N_('Delete %d file(s)?') % count
+        ok_txt = N_('Delete Files')
 
         if qtutils.confirm(title, msg, info_txt, ok_txt,
                            default=True,
@@ -599,11 +651,11 @@ class StatusTreeWidget(QtGui.QTreeWidget):
             items_to_undo = self.modified()
 
         if items_to_undo:
-            if not qtutils.confirm('Revert Unstaged Changes?',
-                                   'This operation drops unstaged changes.'
-                                   '\nThese changes cannot be recovered.',
-                                   'Revert the unstaged changes?',
-                                   'Revert Unstaged Changes',
+            if not qtutils.confirm(N_('Revert Unstaged Changes?'),
+                                   N_('This operation drops unstaged changes.\n'
+                                      'These changes cannot be recovered.'),
+                                   N_('Revert the unstaged changes?'),
+                                   N_('Revert Unstaged Changes'),
                                    default=True,
                                    icon=qtutils.icon('undo.svg')):
                 return
@@ -612,23 +664,23 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                 args.append(self.m.head)
             cmds.do(cmds.Checkout, args + ['--'] + items_to_undo)
         else:
-            msg = self.tr('No files selected for checkout from HEAD.')
+            msg = N_('No files selected for checkout from HEAD.')
             Interaction.log(msg)
 
-    def _revert_uncommitted_edits(self):
-        items_to_undo = self.modified()
+    def _revert_uncommitted_edits(self, items_to_undo):
         if items_to_undo:
-            if not qtutils.confirm('Revert Uncommitted Changes?',
-                                   'This operation drops uncommitted changes.'
-                                   '\nThese changes cannot be recovered.',
-                                   'Revert the uncommitted changes?',
-                                   'Revert Uncommitted Changes',
-                                   default=True,
-                                   icon=qtutils.icon('undo.svg')):
+            if not qtutils.confirm(
+                    N_('Revert Uncommitted Changes?'),
+                    N_('This operation drops uncommitted changes.\n'
+                       'These changes cannot be recovered.'),
+                    N_('Revert the uncommitted changes?'),
+                    N_('Revert Uncommitted Changes'),
+                    default=True,
+                    icon=qtutils.icon('undo.svg')):
                 return
             cmds.do(cmds.Checkout, [self.m.head, '--'] + items_to_undo)
         else:
-            msg = self.tr('No files selected for checkout from HEAD.')
+            msg = N_('No files selected for checkout from HEAD.')
             Interaction.log(msg)
 
     def single_selection(self):
@@ -659,7 +711,7 @@ class StatusTreeWidget(QtGui.QTreeWidget):
                 parent_idx = idx.parent()
                 entry = (parent_idx.row(), idx.row())
             else:
-                entry = (-1, idx.row())
+                entry = (self.idx_header, idx.row())
             result.append(entry)
         return result
 
